package cc.fascinated.bat.command;

import cc.fascinated.bat.features.Feature;
import cc.fascinated.bat.model.BatGuild;
import cc.fascinated.bat.model.BatUser;
import lombok.*;
import net.dv8tion.jda.api.EmbedBuilder;
import net.dv8tion.jda.api.entities.Member;
import net.dv8tion.jda.api.entities.Message;
import net.dv8tion.jda.api.entities.channel.middleman.MessageChannel;
import net.dv8tion.jda.api.interactions.IntegrationType;
import net.dv8tion.jda.api.interactions.InteractionContextType;
import net.dv8tion.jda.api.interactions.commands.OptionMapping;
import net.dv8tion.jda.api.interactions.commands.SlashCommandInteraction;
import net.dv8tion.jda.api.interactions.commands.build.OptionData;
import net.dv8tion.jda.api.interactions.commands.build.SubcommandData;
import net.dv8tion.jda.api.requests.restaction.MessageCreateAction;
import net.dv8tion.jda.internal.interactions.CommandDataImpl;

import java.util.*;

/**
 * @author Braydon
 */
@Getter
public abstract class BatCommand {
    /**
     * The info of this command.
     */
    @NonNull
    private final InternalCommandInfo info;

    /**
     * The feature this command belongs to.
     */
    @Setter
    private Feature feature;

    /**
     * The snowflake of this command, set when
     * this command is registered with Discord.
     */
    @Setter
    private long snowflake;

    /**
     * The sub commands of this command, if any.
     */
    private final Map<String, BatCommand> subCommands = Collections.synchronizedMap(new HashMap<>());

    /**
     * The internal data for this command.
     */
    @Setter(AccessLevel.PRIVATE)
    private CommandDataImpl commandData;

    /**
     * The internal subcommand data for this command.
     */
    @Setter(AccessLevel.PRIVATE)
    private SubcommandData subcommandData;

    public BatCommand() {
        if (!getClass().isAnnotationPresent(CommandInfo.class)) {
            throw new IllegalStateException("Missing @CommandInfo annotation in " + getClass().getSimpleName());
        }
        info = new InternalCommandInfo(getClass().getAnnotation(CommandInfo.class));
        commandData = new CommandDataImpl(info.getName(), info.getDescription())
//                .setDefaultPermissions(DefaultMemberPermissions.enabledFor(info.getPermissions()))
                .setIntegrationTypes(info.isUserInstall() ?
                        EnumSet.of(IntegrationType.USER_INSTALL) :
                        EnumSet.of(IntegrationType.GUILD_INSTALL)
                ).setContexts(info.isGuildOnly() ?
                        EnumSet.of(InteractionContextType.GUILD) :
                        EnumSet.of(InteractionContextType.GUILD, InteractionContextType.BOT_DM, InteractionContextType.PRIVATE_CHANNEL)
                );
    }

    /**
     * Fired when this command is executed.
     *
     * @param guild          the guild the command was executed in, if any
     * @param user           the user who executed the command
     * @param channel        the channel the command was executed in
     * @param member         the member who executed the command, null if not a guild
     * @param commandMessage the message that invoked this command, if any
     * @param arguments      the arguments of the command, if any
     * @param event          the event that invoked this command, if any
     */
    public void execute(BatGuild guild, @NonNull BatUser user, @NonNull MessageChannel channel, Member member,
                        Message commandMessage, String[] arguments, SlashCommandInteraction event) {
    }

    /**
     * Register the given sub commands.
     *
     * @param commands the commands to register
     */
    protected final void addSubCommands(@NonNull BatCommand... commands) {
        for (BatCommand command : commands) {
            // Copy info from the parent command
            if (command.getInfo().getCategory() != info.getCategory()) {
                command.getInfo().setCategory(info.getCategory());
            }
            if (command.getInfo().getPermissions().length == 0) {
                command.getInfo().setPermissions(info.getPermissions());
            }
            if (command.getInfo().isGuildOnly() != info.isGuildOnly()) {
                command.getInfo().setGuildOnly(info.isGuildOnly());
            }
            if (command.getInfo().isBotOwnerOnly() != info.isBotOwnerOnly()) {
                command.getInfo().setBotOwnerOnly(info.isBotOwnerOnly());
            }

            command.setSubcommandData(new SubcommandData(command.getInfo().getName(), command.getInfo().getDescription()));
            for (OptionData option : command.getCommandData().getOptions()) {
                command.getSubcommandData().addOptions(option);
            }
            commandData.addSubcommands(command.getSubcommandData());
            subCommands.put(command.getInfo().getName(), command);
        }
    }

    /**
     * Add the given options
     * to this command.
     *
     * @param options the options to add
     */
    protected final void addOptions(OptionData... options) {
        commandData.addOptions(options);
    }

    /**
     * Get the sub command by its class.
     *
     * @param clazz the class of the sub command
     * @return the sub command
     */
    public BatCommand getSubCommand(Class<? extends BatCommand> clazz) {
        for (Map.Entry<String, BatCommand> entry : subCommands.entrySet()) {
            if (entry.getValue().getClass().equals(clazz)) {
                return entry.getValue();
            }
        }
        return null;
    }

    /**
     * Reply to the message or interaction with the given contents.
     *
     * @param message     the message to reply to, null if interaction
     * @param interaction the interaction to reply to, null if message
     * @param contents    the contents to reply with
     */
    public void replyMessage(Message message, SlashCommandInteraction interaction, String contents) {
        if (message != null) {
            message.reply(contents).queue();
        } else {
            interaction.reply(contents).queue();
        }
    }

    /**
     * Reply to the message or interaction with the given embed.
     *
     * @param message the message to reply to, null if interaction
     * @param interaction the interaction to reply to, null if message
     * @param builder the embed builder to reply with
     */
    public void replyEmbed(Message message, SlashCommandInteraction interaction, EmbedBuilder builder) {
        if (message != null) {
            message.replyEmbeds(builder.build()).queue();
        } else {
            interaction.replyEmbeds(builder.build()).queue();
        }
    }

    /**
     * Get the argument from the command.
     *
     * @param option              the option to get from the slash command
     * @param arguments           the arguments of the invoked command
     * @param argumentIndex       the index of the argument in the command
     * @param concatenateLeftover whether to concatenate the leftover arguments
     * @param event               the event that invoked the command
     * @return the argument
     */
    public Argument getArgument(String option, String[] arguments, int argumentIndex, boolean concatenateLeftover, SlashCommandInteraction event) {
        return new Argument(option, arguments, argumentIndex, concatenateLeftover, event);
    }

    @AllArgsConstructor
    public static class Argument {
        /**
         * The option to get from the slash command.
         */
        private final String option;

        /**
         * The arguments of the invoked command.
         */
        private final String[] arguments;

        /**
         * The index of the argument in the command.
         */
        private final int argumentIndex;

        /**
         * Whether to concatenate the leftover arguments.
         */
        private final boolean concatenateLeftover;

        /**
         * The event that invoked the command.
         */
        private final SlashCommandInteraction event;

        /**
         * Get the argument from the command.
         *
         * @return the argument
         */
        public String getAsString() {
            if (event != null) { // Get the option from the event
                OptionMapping option = event.getOption(this.option);
                if (option == null) {
                    return null;
                }
                return option.getAsString();
            }
            if (arguments.length < argumentIndex) { // Check if the argument index is out of bounds
                return null;
            }
            if (concatenateLeftover) { // Concatenate the leftover arguments
                StringBuilder builder = new StringBuilder();
                for (int i = argumentIndex; i < arguments.length; i++) {
                    builder.append(arguments[i]).append(" ");
                }
                return builder.toString().trim();
            }
            return arguments[argumentIndex]; // Get the argument at the index
        }
    }
}