package cc.fascinated.bat.common;

import cc.fascinated.bat.event.EventListener;
import cc.fascinated.bat.model.BatGuild;
import cc.fascinated.bat.model.BatUser;
import cc.fascinated.bat.service.InteractionService;
import lombok.AllArgsConstructor;
import lombok.Getter;
import lombok.NonNull;
import net.dv8tion.jda.api.entities.emoji.Emoji;
import net.dv8tion.jda.api.events.interaction.component.ButtonInteractionEvent;
import net.dv8tion.jda.api.events.interaction.component.StringSelectInteractionEvent;
import net.dv8tion.jda.api.interactions.components.ActionComponent;
import net.dv8tion.jda.api.interactions.components.ActionRow;
import net.dv8tion.jda.api.interactions.components.ComponentInteraction;
import net.dv8tion.jda.api.interactions.components.ItemComponent;
import net.dv8tion.jda.api.interactions.components.buttons.Button;
import net.dv8tion.jda.api.interactions.components.selections.SelectMenu;
import net.dv8tion.jda.api.interactions.components.selections.SelectOption;
import net.dv8tion.jda.api.interactions.components.selections.StringSelectMenu;
import net.dv8tion.jda.api.utils.data.SerializableData;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.function.Consumer;

import static net.dv8tion.jda.api.interactions.components.Component.Type.STRING_SELECT;

/**
 * @author Fascinated (fascinated7)
 */
@Getter
public class InteractionBuilder {
    /**
     * The button interactions
     */
    private final Map<Button, Consumer<ButtonInteractionEvent>> buttonInteractions = new HashMap<>();

    /**
     * The select menu interactions
     */
    private final Map<SelectOption, Consumer<StringSelectInteractionEvent>> selectMenuInteractions = new HashMap<>();

    /**
     * The select menu id
     */
    private String selectMenuId;

    public InteractionBuilder() {
        InteractionService.INSTANCE.addInteractionBuilder(this);
    }

    /**
     * Adds a button to the interaction builder
     *
     * @param display The display of the button
     * @param onClick The consumer to run when the button is clicked
     * @return - The interaction builder
     */
    public InteractionBuilder addButton(String display, Consumer<ButtonInteractionEvent> onClick) {
        String id = StringUtils.randomString(8);
        this.buttonInteractions.put(Button.primary(id, display), onClick);
        return this;
    }

    /**
     * Adds a URL button to the interaction builder
     *
     * @param display The display of the button
     * @param url The url to open when the button is clicked
     * @return - The interaction builder
     */
    public InteractionBuilder addUrlButton(String display, String url, Emoji emoji) {
        this.buttonInteractions.put(Button.link(url, display).withEmoji(emoji), event -> {});
        return this;
    }

    /**
     * Adds a string selection to the interaction builder
     *
     * @param display the name of the selection
     * @param description the description of the selection
     * @param emoji the emoji of the selection
     * @param onClick the consumer to run when the selection is clicked
     * @return the interaction builder
     */
    public InteractionBuilder addStringSelect(String display, String description, Emoji emoji, Consumer<StringSelectInteractionEvent> onClick) {
        String id = StringUtils.randomString(8);
        this.selectMenuInteractions.put(SelectOption.of(display, id).withDescription(description).withEmoji(emoji), onClick);
        return this;
    }

    /**
     * Builds the interactions into an action row
     *
     * @return The action row
     */
    public List<ActionRow> build() {
        List<ActionRow> components = new ArrayList<>();
        if (!this.getButtonInteractions().isEmpty()) {
            List<Button> buttons = new ArrayList<>(this.getButtonInteractions().keySet());
            components.add(ActionRow.of(buttons));
        }
        if (!this.getSelectMenuInteractions().isEmpty()) {
            List<SelectOption> options = new ArrayList<>(this.getSelectMenuInteractions().keySet());
            String id = StringUtils.randomString(8);
            this.selectMenuId = id;
            components.add(ActionRow.of(StringSelectMenu.create(id)
                    .addOptions(options)
                    .build()));
        }

        return components;
    }
}
