package cc.fascinated.bat.features.autorole.command;

import cc.fascinated.bat.command.BatCommand;
import cc.fascinated.bat.command.CommandInfo;
import cc.fascinated.bat.common.DescriptionBuilder;
import cc.fascinated.bat.common.EmbedUtils;
import cc.fascinated.bat.features.autorole.profile.AutoRoleProfile;
import cc.fascinated.bat.model.BatGuild;
import cc.fascinated.bat.model.BatUser;
import lombok.NonNull;
import lombok.extern.log4j.Log4j2;
import net.dv8tion.jda.api.entities.Guild;
import net.dv8tion.jda.api.entities.Member;
import net.dv8tion.jda.api.entities.Message;
import net.dv8tion.jda.api.entities.Role;
import net.dv8tion.jda.api.entities.channel.middleman.MessageChannel;
import net.dv8tion.jda.api.interactions.commands.SlashCommandInteraction;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;

/**
 * @author Fascinated (fascinated7)
 */
@Log4j2(topic = "AutoRole Sync SubCommand")
@Component("autoroles:sync.sub")
@CommandInfo(name = "sync", description = "Gives everyone their missing auto roles")
public class SyncSubCommand extends BatCommand {
    @Override
    public void execute(BatGuild guild, @NonNull BatUser user, @NonNull MessageChannel channel, Member member, Message commandMessage, String[] arguments, SlashCommandInteraction event) {
        AutoRoleProfile profile = guild.getProfile(AutoRoleProfile.class);
        if (profile.getRoles().isEmpty()) {
            event.replyEmbeds(EmbedUtils.errorEmbed()
                    .setDescription("There are no auto roles set")
                    .build()).queue();
            return;
        }

        Guild discordGuild = guild.getDiscordGuild();
        event.replyEmbeds(EmbedUtils.genericEmbed()
                        .setDescription("Finding members that are missing auto roles...")
                        .build())
                .queue(message -> {
                    log.info("Finding members that are missing auto roles in guild \"{}\"", discordGuild.getName());
                    List<Member> members = discordGuild.loadMembers().get();
                    Map<Role, Integer> rolesGiven = new HashMap<>();

                    // Find members that are missing the auto roles
                    members.removeIf(foundMember -> {
                        if (foundMember.getUser().isBot() || foundMember.getId().equals(discordGuild.getSelfMember().getId())) {
                            return true;
                        }
                        return new HashSet<>(foundMember.getRoles()).containsAll(profile.getRoles());
                    });
                    log.info("Found {} members that are missing auto roles in guild \"{}\"", members.size(), discordGuild.getName());

                    // No members were missing roles
                    if (members.isEmpty()) {
                        message.editOriginalEmbeds(EmbedUtils.successEmbed()
                                .setDescription("There are no members missing auto roles")
                                .build()).queue();
                        return;
                    }

                    int finished = 0;
                    for (Member foundMember : members) {
                        // Check if the user doesn't have the role, so we can
                        // show the incremented count when we're done
                        for (Role role : profile.getRoles()) {
                            if (foundMember.getRoles().contains(role)) {
                                continue;
                            }
                            rolesGiven.put(role, rolesGiven.getOrDefault(role, 0) + 1);
                        }
                        // Give the user the roles
                        discordGuild.modifyMemberRoles(foundMember, profile.getRoles(), null).queue();
                        finished++;

                        // Update the message every 10 members
                        if (finished % 10 == 0 || finished == 1) {
                            message.editOriginalEmbeds(EmbedUtils.genericEmbed()
                                    .setDescription("""
                                            Giving auto roles...
                                            Completed: `%s`/`%s`
                                            """.formatted(finished, members.size()))
                                    .build()).queue();
                        }

                        // We're finished giving all the roles
                        if (finished == members.size()) {
                            DescriptionBuilder description = new DescriptionBuilder(
                                    "Successfully gave auto roles to `%s` member%s".formatted(
                                            members.size(),
                                            members.size() == 1 ? "" : "s"
                                    ));
                            description.emptyLine();
                            description.appendLine("**Auto Roles Given:**", false);
                            for (Map.Entry<Role, Integer> entry : rolesGiven.entrySet()) {
                                description.appendLine("%s: %s member%s".formatted(
                                        entry.getKey().getAsMention(),
                                        entry.getValue(),
                                        entry.getValue() == 1 ? "" : "s"
                                ), true);
                            }
                            message.editOriginalEmbeds(EmbedUtils.successEmbed().setDescription(description.build()).build()).queue();
                        }
                    }
                });
    }
}
