package cc.fascinated.bat.features.logging.listeners;

import cc.fascinated.bat.common.DescriptionBuilder;
import cc.fascinated.bat.common.EmbedUtils;
import cc.fascinated.bat.common.EnumUtils;
import cc.fascinated.bat.common.TimeUtils;
import cc.fascinated.bat.event.EventListener;
import cc.fascinated.bat.features.logging.LogFeature;
import cc.fascinated.bat.features.logging.LogType;
import cc.fascinated.bat.model.BatGuild;
import cc.fascinated.bat.model.BatUser;
import lombok.NonNull;
import lombok.extern.log4j.Log4j2;
import net.dv8tion.jda.api.entities.Guild;
import net.dv8tion.jda.api.entities.Invite;
import net.dv8tion.jda.api.entities.channel.ChannelType;
import net.dv8tion.jda.api.entities.channel.concrete.TextChannel;
import net.dv8tion.jda.api.entities.channel.concrete.VoiceChannel;
import net.dv8tion.jda.api.events.guild.invite.GuildInviteCreateEvent;
import net.dv8tion.jda.api.events.guild.update.*;
import net.dv8tion.jda.api.interactions.DiscordLocale;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Component;

/**
 * @author Fascinated (fascinated7)
 */
@Component
@Log4j2(topic = "Logging - Guild Listener")
public class GuildListener implements EventListener {
    private final LogFeature logFeature;

    @Autowired
    public GuildListener(@NonNull ApplicationContext context) {
        this.logFeature = context.getBean(LogFeature.class);
    }

    @Override
    public void onGuildInviteCreate(@NonNull BatGuild guild, @NonNull Invite invite, @NonNull GuildInviteCreateEvent event) {
        log.info("Invite created in guild \"{}\" with code \"{}\"", guild.getGuild().getName(), invite.getCode());

        invite.expand().queue(expandedInvite -> {
            DescriptionBuilder description = new DescriptionBuilder("Invite Created");
            description.appendLine("Invite: %s".formatted(expandedInvite.getUrl()), true);
            if (expandedInvite.getChannel() != null) {
                description.appendLine("Channel: %s".formatted(
                        expandedInvite.getChannel().getType() == ChannelType.TEXT ?
                                "<#%s>".formatted(expandedInvite.getChannel().getId()) : expandedInvite.getChannel().getName()
                ), true);
            }
            description.appendLine("Creator: %s".formatted(expandedInvite.getInviter() == null ? "Unknown" : expandedInvite.getInviter().getAsMention()), true);
            description.appendLine("Max Uses: `%s`".formatted(expandedInvite.getMaxUses() == 0 ? "Infinite" : expandedInvite.getMaxUses()), true);
            ;
            logFeature.sendLog(guild, LogType.INVITE_CREATE, EmbedUtils.successEmbed().setDescription(description.build()).build());
        });
    }

    @Override
    public void onGuildUpdateName(@NonNull BatGuild guild, String oldName, String newName, @NonNull GuildUpdateNameEvent event) {
        log.info("Guild name was updated in guild \"{}\" from \"{}\" to \"{}\"", guild.getGuild().getName(), oldName, newName);

        DescriptionBuilder description = new DescriptionBuilder("Guild Name Updated")
                .appendLine("Name: `%s` -> `%s`".formatted(oldName, newName), true);
        logFeature.sendLog(guild, LogType.GUILD_CONFIGURATION, EmbedUtils.successEmbed().setDescription(description.build()).build());
    }

    @Override
    public void onGuildUpdateAfkChannel(@NonNull BatGuild guild, VoiceChannel oldChannel, VoiceChannel newChannel, @NonNull GuildUpdateAfkChannelEvent event) {
        log.info("Guild AFK channel was updated in guild \"{}\" from \"{}\" to \"{}\"", guild.getGuild().getName(), oldChannel == null ? "None" : oldChannel.getName(), newChannel == null ? "None" : newChannel.getName());

        DescriptionBuilder description = new DescriptionBuilder("Guild AFK Channel Updated")
                .appendLine("Channel: `%s` -> `%s`".formatted(
                        oldChannel == null ? "None" : oldChannel.getName(),
                        newChannel == null ? "None" : newChannel.getName()
                ), true);
        logFeature.sendLog(guild, LogType.GUILD_CONFIGURATION, EmbedUtils.successEmbed().setDescription(description.build()).build());
    }

    @Override
    public void onGuildUpdateAfkTimeout(@NonNull BatGuild guild, Guild.Timeout oldTimeout, Guild.Timeout newTimeout, @NonNull GuildUpdateAfkTimeoutEvent event) {
        log.info("Guild AFK timeout was updated in guild \"{}\" from \"{}\" to \"{}\"",
                guild.getGuild().getName(), oldTimeout.getSeconds(), newTimeout.getSeconds());

        DescriptionBuilder description = new DescriptionBuilder("Guild AFK Timeout Updated")
                .appendLine("Timeout: `%s` -> `%s`".formatted(
                        TimeUtils.format(oldTimeout.getSeconds() * 1000L),
                        TimeUtils.format(newTimeout.getSeconds() * 1000L)
                ), true);
        logFeature.sendLog(guild, LogType.GUILD_CONFIGURATION, EmbedUtils.successEmbed().setDescription(description.build()).build());
    }

    @Override
    public void onGuildUpdateBanner(@NonNull BatGuild guild, String oldBannerUrl, String newBannerUrl, @NonNull GuildUpdateBannerEvent event) {
        log.info("Guild banner was updated in guild \"{}\" from \"{}\" to \"{}\"", guild.getGuild().getName(), oldBannerUrl, newBannerUrl);

        DescriptionBuilder description = new DescriptionBuilder("Guild Banner Updated")
                .appendLine("Banner: [click here](%s)".formatted(newBannerUrl), true);
        logFeature.sendLog(guild, LogType.GUILD_CONFIGURATION, EmbedUtils.successEmbed().setDescription(description.build()).build());
    }

    @Override
    public void onGuildUpdateDescription(@NonNull BatGuild guild, String oldDescription, String newDescription, @NonNull GuildUpdateDescriptionEvent event) {
        log.info("Guild description was updated in guild \"{}\" from \"{}\" to \"{}\"", guild.getGuild().getName(), oldDescription, newDescription);

        DescriptionBuilder description = new DescriptionBuilder("Guild Description Updated")
                .appendLine("Description: `%s` -> `%s`".formatted(oldDescription, newDescription), true);
        logFeature.sendLog(guild, LogType.GUILD_CONFIGURATION, EmbedUtils.successEmbed().setDescription(description.build()).build());
    }

    @Override
    public void onGuildUpdateIcon(@NonNull BatGuild guild, String oldIconUrl, String newIconUrl, @NonNull GuildUpdateIconEvent event) {
        log.info("Guild icon was updated in guild \"{}\" from \"{}\" to \"{}\"", guild.getGuild().getName(), oldIconUrl, newIconUrl);

        DescriptionBuilder description = new DescriptionBuilder("Guild Icon Updated")
                .appendLine("Icon: [click here](%s)".formatted(newIconUrl), true);
        logFeature.sendLog(guild, LogType.GUILD_CONFIGURATION, EmbedUtils.successEmbed().setDescription(description.build()).build());
    }

    @Override
    public void onGuildUpdateLocale(@NonNull BatGuild guild, DiscordLocale oldLocale, DiscordLocale newLocale, @NonNull GuildUpdateLocaleEvent event) {
        log.info("Guild primary language was updated in guild \"{}\" from \"{}\" to \"{}\"", guild.getGuild().getName(), oldLocale, newLocale);

        DescriptionBuilder description = new DescriptionBuilder("Guild Primary Language Updated")
                .appendLine("Language: `%s` -> `%s`".formatted(
                        oldLocale.getLanguageName(),
                        newLocale.getLanguageName()
                ), true);
        logFeature.sendLog(guild, LogType.GUILD_CONFIGURATION, EmbedUtils.successEmbed().setDescription(description.build()).build());
    }

    @Override
    public void onGuildUpdateCommunityUpdatesChannel(@NonNull BatGuild guild, TextChannel oldChannel, TextChannel newChannel, @NonNull GuildUpdateCommunityUpdatesChannelEvent event) {
        log.info("Guild community updates channel was updated in guild \"{}\" from \"{}\" to \"{}\"", guild.getGuild().getName(), oldChannel == null ? "None" : oldChannel.getName(), newChannel == null ? "None" : newChannel.getName());

        DescriptionBuilder description = new DescriptionBuilder("Community Updates Channel Updated")
                .appendLine("Channel: `%s` -> `%s`".formatted(
                        oldChannel == null ? "None" : oldChannel.getName(),
                        newChannel == null ? "None" : newChannel.getName()
                ), true);
        logFeature.sendLog(guild, LogType.GUILD_CONFIGURATION, EmbedUtils.successEmbed().setDescription(description.build()).build());
    }


    @Override
    public void onGuildUpdateBoostTier(@NonNull BatGuild guild, Guild.BoostTier oldTier, Guild.BoostTier newTier, @NonNull GuildUpdateBoostTierEvent event) {
        log.info("Guild boost tier was updated in guild \"{}\" from \"{}\" to \"{}\"", guild.getGuild().getName(), oldTier.getKey(), newTier.getKey());

        DescriptionBuilder description = new DescriptionBuilder("Guild Boost Tier Updated")
                .appendLine("Tier: `%s` -> `%s`".formatted(oldTier.getKey(), newTier.getKey()), true);
        logFeature.sendLog(guild, LogType.BOOST_TIER_UPDATED, EmbedUtils.successEmbed().setDescription(description.build()).build());
    }

    @Override
    public void onGuildUpdateMaxMembers(@NonNull BatGuild guild, int oldCount, int newCount, @NonNull GuildUpdateMaxMembersEvent event) {
        log.info("Guild max members was updated in guild \"{}\" from \"{}\" to \"{}\"", guild.getGuild().getName(), oldCount, newCount);

        DescriptionBuilder description = new DescriptionBuilder("Guild Max Members Updated")
                .appendLine("Members: `%s` -> `%s`".formatted(oldCount, newCount), true);
        logFeature.sendLog(guild, LogType.GUILD_CONFIGURATION, EmbedUtils.successEmbed().setDescription(description.build()).build());
    }

    @Override
    public void onGuildUpdateExplicitContentLevel(@NonNull BatGuild guild, Guild.ExplicitContentLevel oldLevel, Guild.ExplicitContentLevel newLevel, @NonNull GuildUpdateExplicitContentLevelEvent event) {
        log.info("Guild explicit content level was updated in guild \"{}\" from \"{}\" to \"{}\"", guild.getGuild().getName(), oldLevel.getKey(), newLevel.getKey());

        DescriptionBuilder description = new DescriptionBuilder("Guild Explicit Content Level Updated")
                .appendLine("Level: `%s` -> `%s`".formatted(
                        EnumUtils.getEnumName(oldLevel),
                        EnumUtils.getEnumName(newLevel)
                ), true);
        logFeature.sendLog(guild, LogType.GUILD_CONFIGURATION, EmbedUtils.successEmbed().setDescription(description.build()).build());
    }

    @Override
    public void onGuildUpdateMFALevel(@NonNull BatGuild guild, Guild.MFALevel oldLevel, Guild.MFALevel newLevel, @NonNull GuildUpdateMFALevelEvent event) {
        log.info("Guild MFA level was updated in guild \"{}\" from \"{}\" to \"{}\"", guild.getGuild().getName(), oldLevel.getKey(), newLevel.getKey());

        DescriptionBuilder description = new DescriptionBuilder("Guild MFA Level Updated")
                .appendLine("Level: `%s` -> `%s`".formatted(
                        EnumUtils.getEnumName(oldLevel),
                        EnumUtils.getEnumName(newLevel)
                ), true);
        logFeature.sendLog(guild, LogType.GUILD_CONFIGURATION, EmbedUtils.successEmbed().setDescription(description.build()).build());
    }

    @Override
    public void onGuildUpdateNotificationLevel(@NonNull BatGuild guild, Guild.NotificationLevel oldLevel, Guild.NotificationLevel newLevel, @NonNull GuildUpdateNotificationLevelEvent event) {
        log.info("Guild notification level was updated in guild \"{}\" from \"{}\" to \"{}\"", guild.getGuild().getName(), oldLevel.getKey(), newLevel.getKey());

        DescriptionBuilder description = new DescriptionBuilder("Guild Notification Level Updated")
                .appendLine("Level: `%s` -> `%s`".formatted(
                        EnumUtils.getEnumName(oldLevel),
                        EnumUtils.getEnumName(newLevel)
                ), true);
        logFeature.sendLog(guild, LogType.GUILD_CONFIGURATION, EmbedUtils.successEmbed().setDescription(description.build()).build());
    }

    @Override
    public void onGuildUpdateNSFWLevel(@NonNull BatGuild guild, Guild.NSFWLevel oldLevel, Guild.NSFWLevel newLevel, @NonNull GuildUpdateNSFWLevelEvent event) {
        log.info("Guild NSFW level was updated in guild \"{}\" from \"{}\" to \"{}\"", guild.getGuild().getName(), oldLevel.getKey(), newLevel.getKey());

        DescriptionBuilder description = new DescriptionBuilder("Guild NSFW Level Updated")
                .appendLine("Level: `%s` -> `%s`".formatted(
                        EnumUtils.getEnumName(oldLevel),
                        EnumUtils.getEnumName(newLevel)
                ), true);
        logFeature.sendLog(guild, LogType.GUILD_CONFIGURATION, EmbedUtils.successEmbed().setDescription(description.build()).build());
    }

    @Override
    public void onGuildUpdateOwner(@NonNull BatGuild guild, @NonNull BatUser oldOwner, @NonNull BatUser newOwner, @NonNull GuildUpdateOwnerEvent event) {
        log.info("Guild owner was updated in guild \"{}\" from \"{}\" to \"{}\"", guild.getGuild().getName(), oldOwner.getUser().getAsTag(), newOwner.getUser().getAsTag());

        DescriptionBuilder description = new DescriptionBuilder("Guild Owner Updated")
                .appendLine("Owner: %s -> %s".formatted(oldOwner.getUser().getAsMention(), newOwner.getUser().getAsMention()), true);
        logFeature.sendLog(guild, LogType.GUILD_CONFIGURATION, EmbedUtils.successEmbed().setDescription(description.build()).build());
    }

    @Override
    public void onGuildUpdateRulesChannel(@NonNull BatGuild guild, TextChannel oldChannel, TextChannel newChannel, @NonNull GuildUpdateRulesChannelEvent event) {
        log.info("Guild rules channel was updated in guild \"{}\" from \"{}\" to \"{}\"", guild.getGuild().getName(), oldChannel == null ? "None" : oldChannel.getName(), newChannel == null ? "None" : newChannel.getName());

        DescriptionBuilder description = new DescriptionBuilder("Rules Channel Updated")
                .appendLine("Channel: %s -> %s".formatted(
                        oldChannel == null ? "`None`" : oldChannel.getAsMention(),
                        newChannel == null ? "`None`" : newChannel.getAsMention()
                ), true);
        logFeature.sendLog(guild, LogType.GUILD_CONFIGURATION, EmbedUtils.successEmbed().setDescription(description.build()).build());
    }

    @Override
    public void onGuildUpdateSystemChannel(@NonNull BatGuild guild, TextChannel oldChannel, TextChannel newChannel, @NonNull GuildUpdateSystemChannelEvent event) {
        log.info("Guild system channel was updated in guild \"{}\" from \"{}\" to \"{}\"", guild.getGuild().getName(), oldChannel == null ? "None" : oldChannel.getName(), newChannel == null ? "None" : newChannel.getName());

        DescriptionBuilder description = new DescriptionBuilder("System Channel Updated")
                .appendLine("Channel: %s -> %s".formatted(
                        oldChannel == null ? "`None`" : oldChannel.getAsMention(),
                        newChannel == null ? "`None`" : newChannel.getAsMention()
                ), true);
        logFeature.sendLog(guild, LogType.GUILD_CONFIGURATION, EmbedUtils.successEmbed().setDescription(description.build()).build());
    }

    @Override
    public void onGuildUpdateVanityCode(@NonNull BatGuild guild, String oldCode, String newCode, @NonNull GuildUpdateVanityCodeEvent event) {
        log.info("Guild vanity code was updated in guild \"{}\" from \"{}\" to \"{}\"", guild.getGuild().getName(), oldCode, newCode);

        DescriptionBuilder description = new DescriptionBuilder("Vanity Code Updated")
                .appendLine("Code: `%s` -> `%s`".formatted(
                        oldCode == null ? "None" : oldCode,
                        newCode == null ? "None" : newCode
                ), true);
        logFeature.sendLog(guild, LogType.GUILD_CONFIGURATION, EmbedUtils.successEmbed().setDescription(description.build()).build());
    }

    @Override
    public void onGuildUpdateVerificationLevel(@NonNull BatGuild guild, Guild.VerificationLevel oldLevel, Guild.VerificationLevel newLevel, @NonNull GuildUpdateVerificationLevelEvent event) {
        log.info("Guild verification level was updated in guild \"{}\" from \"{}\" to \"{}\"", guild.getGuild().getName(), oldLevel.getKey(), newLevel.getKey());

        DescriptionBuilder description = new DescriptionBuilder("Verification Level Updated")
                .appendLine("Level: `%s` -> `%s`".formatted(
                        EnumUtils.getEnumName(oldLevel),
                        EnumUtils.getEnumName(newLevel)
                ), true);
        logFeature.sendLog(guild, LogType.GUILD_CONFIGURATION, EmbedUtils.successEmbed().setDescription(description.build()).build());
    }

    @Override
    public void onGuildUpdateSplash(@NonNull BatGuild guild, String oldSplashUrl, String newSplashUrl, @NonNull GuildUpdateSplashEvent event) {
        log.info("Guild splash was updated in guild \"{}\" from \"{}\" to \"{}\"", guild.getGuild().getName(), oldSplashUrl, newSplashUrl);

        DescriptionBuilder description = new DescriptionBuilder("Guild Splash Updated")
                .appendLine("Splash: [click here](%s)".formatted(newSplashUrl), true);
        logFeature.sendLog(guild, LogType.GUILD_CONFIGURATION, EmbedUtils.successEmbed().setDescription(description.build()).build());
    }
}
