package cc.fascinated.bat.features.logging.listeners;

import cc.fascinated.bat.common.DescriptionBuilder;
import cc.fascinated.bat.common.EmbedUtils;
import cc.fascinated.bat.common.RoleUtils;
import cc.fascinated.bat.event.EventListener;
import cc.fascinated.bat.features.logging.LogFeature;
import cc.fascinated.bat.features.logging.LogType;
import cc.fascinated.bat.model.BatGuild;
import lombok.NonNull;
import lombok.extern.log4j.Log4j2;
import net.dv8tion.jda.api.Permission;
import net.dv8tion.jda.api.entities.Role;
import net.dv8tion.jda.api.entities.RoleIcon;
import net.dv8tion.jda.api.events.role.RoleCreateEvent;
import net.dv8tion.jda.api.events.role.RoleDeleteEvent;
import net.dv8tion.jda.api.events.role.update.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.awt.*;
import java.util.EnumSet;

/**
 * @author Fascinated (fascinated7)
 */
@Component
@Log4j2(topic = "Logging - Role Listener")
public class RoleListener implements EventListener {
    private final LogFeature logFeature;

    @Autowired
    public RoleListener(@NonNull LogFeature logFeature) {
        this.logFeature = logFeature;
    }

    @Override
    public void onRoleUpdatePermissions(@NonNull BatGuild guild, @NonNull RoleUpdatePermissionsEvent event) {
        log.info("Role \"{}\" permissions were updated in guild \"{}\"", event.getRole().getName(), guild.getName());
        EnumSet<Permission> oldPermissions = event.getOldValue();
        EnumSet<Permission> newPermissions = event.getNewValue();

        StringBuilder allowedPermissions = new StringBuilder();
        for (Permission permission : newPermissions) {
            if (!oldPermissions.contains(permission)) {
                allowedPermissions.append("`").append(permission.getName()).append("`, ");
            }
        }
        StringBuilder deniedPermissions = new StringBuilder();
        for (Permission permission : oldPermissions) {
            if (!newPermissions.contains(permission)) {
                deniedPermissions.append("`").append(permission.getName()).append("`, ");
            }
        }

        DescriptionBuilder description = new DescriptionBuilder("Role Permissions Updated")
                .appendLine("Role: %s".formatted(event.getRole().getAsMention()), true);
        if (!allowedPermissions.isEmpty()) {
            description.appendLine("Allowed Permissions: %s".formatted(allowedPermissions.substring(0, allowedPermissions.length() - 2)), true);
        }
        if (!deniedPermissions.isEmpty()) {
            description.appendLine("Denied Permissions: %s".formatted(deniedPermissions.substring(0, deniedPermissions.length() - 2)), true);
        }

        logFeature.sendLog(guild, LogType.ROLE_CONFIGURATION, EmbedUtils.successEmbed()
                .setDescription(description.build())
                .build());
    }

    @Override
    public void onRoleCreate(@NonNull BatGuild guild, @NonNull RoleCreateEvent event) {
        Role role = event.getRole();
        log.info("Role \"{}\" was created in guild \"{}\"", role.getName(), guild.getName());
        logFeature.sendLog(guild, LogType.ROLE_CREATE, EmbedUtils.successEmbed()
                .setDescription(new DescriptionBuilder("Role Created")
                        .appendLine("Role: %s".formatted(role.getAsMention()), true)
                        .appendLine("Color: %s".formatted(RoleUtils.getFormattedColor(role)), true)
                        .appendLine("Permissions: %s".formatted(RoleUtils.getFormattedPermissions(role)), true)
                        .build())
                .build());
    }

    @Override
    public void onRoleDelete(@NonNull BatGuild guild, @NonNull RoleDeleteEvent event) {
        Role role = event.getRole();
        log.info("Role \"{}\" was deleted in guild \"{}\"", role.getName(), guild.getName());
        logFeature.sendLog(guild, LogType.ROLE_DELETE, EmbedUtils.successEmbed()
                .setDescription(new DescriptionBuilder("Role Deleted")
                        .appendLine("Role: `%s`".formatted(role.getName()), true)
                        .appendLine("Color: %s".formatted(RoleUtils.getFormattedColor(role)), true)
                        .appendLine("Permissions: %s".formatted(RoleUtils.getFormattedPermissions(role)), true)
                        .build())
                .build());
    }

    @Override
    public void onRoleUpdateName(@NonNull BatGuild guild, @NonNull RoleUpdateNameEvent event) {
        String oldName = event.getOldName();
        String newName = event.getNewName();
        log.info("Role \"{}\" was renamed to \"{}\" in guild \"{}\"", oldName, newName, guild.getName());
        logFeature.sendLog(guild, LogType.ROLE_CONFIGURATION, EmbedUtils.successEmbed()
                .setDescription(new DescriptionBuilder("Role Renamed")
                        .appendLine("Role: %s".formatted(event.getRole().getAsMention()), true)
                        .appendLine("Name: `%s` -> `%s`".formatted(oldName, newName), true)
                        .build())
                .build());
    }

    @Override
    public void onRoleUpdateColor(@NonNull BatGuild guild, @NonNull RoleUpdateColorEvent event) {
        Color oldColor = event.getOldColor();
        Color newColor = event.getNewColor();

        log.info("Role \"{}\" color was updated in guild \"{}\"", event.getRole().getName(), guild.getName());
        logFeature.sendLog(guild, LogType.ROLE_CONFIGURATION, EmbedUtils.successEmbed()
                .setDescription(new DescriptionBuilder("Role Color Updated")
                        .appendLine("Role: %s".formatted(event.getRole().getAsMention()), true)
                        .appendLine("Color: %s -> %s".formatted(
                                RoleUtils.getFormattedColor(oldColor),
                                RoleUtils.getFormattedColor(newColor)
                        ), true)
                        .build())
                .build());
    }

    @Override
    public void onRoleUpdateIcon(@NonNull BatGuild guild, @NonNull RoleUpdateIconEvent event) {
        RoleIcon oldIcon = event.getOldIcon();
        RoleIcon newIcon = event.getNewIcon();

        log.info("Role \"{}\" icon was updated in guild \"{}\"", event.getRole().getName(), guild.getName());
        logFeature.sendLog(guild, LogType.ROLE_CONFIGURATION, EmbedUtils.successEmbed()
                .setDescription(new DescriptionBuilder("Role Icon Updated")
                        .appendLine("Role: %s".formatted(event.getRole().getAsMention()), true)
                        .appendLine("Icon: `%s` -> `%s`".formatted(
                                oldIcon == null ? "None" : oldIcon.getEmoji(),
                                newIcon == null ? "None" : newIcon.getEmoji()
                        ), true)
                        .build())
                .build());
    }

    @Override
    public void onRoleUpdateHoisted(@NonNull BatGuild guild, @NonNull Role role, boolean wasHoisted, boolean isHoisted, @NonNull RoleUpdateHoistedEvent event) {
        log.info("Role \"{}\" hoisted status was updated in guild \"{}\"", role.getName(), guild.getName());
        logFeature.sendLog(guild, LogType.ROLE_CONFIGURATION, EmbedUtils.successEmbed()
                .setDescription(new DescriptionBuilder("Role Hoisted Status Updated")
                        .appendLine("Role: %s".formatted(role.getAsMention()), true)
                        .appendLine("Hoisted: `%s` -> `%s`".formatted(
                                wasHoisted ? "Yes" : "No",
                                isHoisted ? "Yes" : "No"
                        ), true)
                        .build())
                .build());
    }
}
