package cc.fascinated.bat.features.moderation;

import cc.fascinated.bat.event.EventListener;
import cc.fascinated.bat.features.Feature;
import cc.fascinated.bat.features.FeatureProfile;
import cc.fascinated.bat.features.moderation.command.*;
import cc.fascinated.bat.features.moderation.punish.Punishment;
import cc.fascinated.bat.features.moderation.punish.PunishmentProfile;
import cc.fascinated.bat.model.BatGuild;
import cc.fascinated.bat.model.BatUser;
import cc.fascinated.bat.service.CommandService;
import cc.fascinated.bat.service.DiscordService;
import cc.fascinated.bat.service.GuildService;
import cc.fascinated.bat.service.UserService;
import lombok.NonNull;
import lombok.extern.log4j.Log4j2;
import net.dv8tion.jda.api.entities.Guild;
import net.dv8tion.jda.api.entities.channel.ChannelType;
import net.dv8tion.jda.api.entities.channel.unions.ChannelUnion;
import net.dv8tion.jda.api.events.channel.ChannelCreateEvent;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.time.Instant;
import java.util.List;
import java.util.Map;

/**
 * @author Fascinated (fascinated7)
 */
@Component
@Log4j2
public class ModerationFeature extends Feature implements EventListener {
    private final GuildService guildService;
    private final UserService userService;

    @Autowired
    public ModerationFeature(@NonNull ApplicationContext context, @NonNull CommandService commandService, @NonNull GuildService guildService,
                             @NonNull UserService userService) {
        super("Moderation", FeatureProfile.FeatureState.DISABLED, true);
        this.guildService = guildService;
        this.userService = userService;

        super.registerCommand(commandService, context.getBean(PurgeCommand.class));
        super.registerCommand(commandService, context.getBean(WarnCommand.class));
        super.registerCommand(commandService, context.getBean(KickCommand.class));
        super.registerCommand(commandService, context.getBean(BanCommand.class));
        super.registerCommand(commandService, context.getBean(MuteCommand.class));
        super.registerCommand(commandService, context.getBean(UnbanCommand.class));
        super.registerCommand(commandService, context.getBean(UnmuteCommand.class));
        super.registerCommand(commandService, context.getBean(PunishHistoryCommand.class));
    }

    /**
     * This method is called every minute to check
     * the status of temporary punishments
     */
    @Scheduled(cron = "0 * * * * *")
    public void checkTemporaryPunishments() {
        for (Guild guild : DiscordService.JDA.getGuilds()) {
            BatGuild batGuild = guildService.getGuild(guild.getId());
            if (batGuild.getFeatureProfile().isFeatureDisabled(this)) { // Check if the feature is disabled
                continue;
            }

            PunishmentProfile profile = batGuild.getPunishmentProfile();

            for (Map.Entry<String, List<Punishment>> entry : profile.getPunishments().entrySet()) {
                BatUser user = userService.getUser(entry.getKey());
                if (user == null) { // This should never happen
                    continue;
                }

                for (Punishment punishment : entry.getValue()) {
                    // Check if the punishment is not temporary and has not expired
                    if (punishment.getEndDate() == null
                            || punishment.isExpired()
                            || punishment.getEndDate().toInstant().isAfter(Instant.now())) {
                        continue;
                    }
                    BatUser issuer = userService.getUser(punishment.getIssuer());
                    punishment.setExpired(true);
                    profile.removePunishment(
                            user,
                            issuer,
                            batGuild,
                            punishment.getType(),
                            punishment.getReason()
                    );
                    log.info("Expired punishment for user {}", user.getDiscordUser().getEffectiveName());
                }
            }
        }
    }

    @Override
    public void onChannelCreate(@NonNull BatGuild guild, @NonNull ChannelCreateEvent event) {
        if (guild.getFeatureProfile().isFeatureDisabled(this)) { // Check if the feature is disabled
            return;
        }

        PunishmentProfile profile = guild.getPunishmentProfile();
        ChannelUnion channelUnion = event.getChannel();
        if (channelUnion.getType() != ChannelType.TEXT) {
            return;
        }
        profile.upsetMutedRolePermissions(guild, channelUnion.asGuildChannel());
    }
}
