package cc.fascinated.bat.features.moderation.punish;

import cc.fascinated.bat.common.*;
import cc.fascinated.bat.features.logging.LogFeature;
import cc.fascinated.bat.features.logging.LogType;
import cc.fascinated.bat.model.BatGuild;
import cc.fascinated.bat.model.BatUser;
import com.google.gson.Gson;
import lombok.Getter;
import net.dv8tion.jda.api.Permission;
import net.dv8tion.jda.api.entities.Guild;
import net.dv8tion.jda.api.entities.Member;
import net.dv8tion.jda.api.entities.PermissionOverride;
import net.dv8tion.jda.api.entities.Role;
import net.dv8tion.jda.api.entities.channel.concrete.TextChannel;
import net.dv8tion.jda.api.entities.channel.concrete.VoiceChannel;
import net.dv8tion.jda.api.entities.channel.middleman.GuildChannel;
import net.dv8tion.jda.api.interactions.commands.SlashCommandInteraction;
import org.bson.Document;

import java.util.*;
import java.util.concurrent.TimeUnit;

/**
 * @author Fascinated (fascinated7)
 */
@Getter
public class PunishmentProfile extends Serializable {
    /**
     * The punishments issued in this guild
     */
    private final Map<String, List<Punishment>> punishments = new HashMap<>();

    /**
     * Punish a member
     *
     * @param user         The user to punish
     * @param issuer       The issuer of the punishment
     * @param guild        The guild the punishment is being issued in
     * @param type         The type of punishment
     * @param reasonString The reason for the punishment
     * @param length       The length of the punishment
     * @return The punishment issued
     */
    private Punishment punishMember(BatUser user, BatUser issuer, BatGuild guild, PunishmentType type, String reasonString, long length) {
        Punishment punishment = new Punishment(
                type,
                reasonString,
                issuer.getId(),
                new Date(),
                length == -1 ? null : new Date(System.currentTimeMillis() + length),
                false
        );
        List<Punishment> userPunishments = this.punishments.getOrDefault(user.getId(), new ArrayList<>());

        Punishment previousPunishment = this.getLatestActivePunishment(user, type);
        // Expire the previous punishment
        if (previousPunishment != null &&
                (previousPunishment.getType() == PunishmentType.MUTE || previousPunishment.getType() == PunishmentType.BAN)) {
            previousPunishment.setExpired(true);
        }
        userPunishments.add(punishment);
        this.punishments.put(user.getId(), userPunishments);

        String reason = reasonString == null ? "No reason provided" : reasonString;
        String lengthFormatted = TimeUtils.format(length);

        Guild discordGuild = guild.getDiscordGuild();
        DescriptionBuilder punishmentDescription = new DescriptionBuilder("%s Issued".formatted(EnumUtils.getEnumName(type)))
                .appendLine("User: %s".formatted(user.getDiscordUser().getAsMention()), true)
                .appendLine("Issuer: %s".formatted(issuer.getDiscordUser().getAsMention()), true)
                .appendLine("Reason: `%s`".formatted(reason), true);
        // Only applicable for Mutes and Bans
        if (type == PunishmentType.MUTE || type == PunishmentType.BAN) {
            punishmentDescription.appendLine("Length: `%s`".formatted(length == -1 ? "Permanent" : lengthFormatted), true);
        }
        LogFeature.INSTANCE.sendLog(guild, LogType.PUNISHMENT_ISSUED, EmbedUtils.successEmbed()
                .setDescription(punishmentDescription.build())
                .build());

        user.getDiscordUser().openPrivateChannel().queue(channel -> {
            DescriptionBuilder descriptionBuilder = new DescriptionBuilder(null);
            descriptionBuilder.appendLine("🛡️ You have been **%s** in **%s**".formatted(
                    type.getPastTense(),
                    discordGuild.getName()
            ), false);
            if (length != -1 && (type == PunishmentType.MUTE || type == PunishmentType.BAN)) {
                descriptionBuilder.appendLine("Length: `%s`".formatted(lengthFormatted), true);
            }
            descriptionBuilder.appendLine("Reason: `%s`".formatted(reason), true);
            try {
                channel.sendMessage(descriptionBuilder.build()).complete();
            } catch (Exception ignored) {
            } // Ignore since the user has DMs disabled
        });

        switch (type) {
            case BAN -> discordGuild.ban(user.getDiscordUser(), 0, TimeUnit.SECONDS).queue();
            case KICK -> discordGuild.kick(user.getDiscordUser()).queue();
            case MUTE -> {
                this.ensureMutedPermissionsAreSetup(guild); // Ensure the muted role permissions are set up
                Role mutedRole = this.findOrCreateMutedRole(guild);
                discordGuild.addRoleToMember(user.getDiscordUser(), mutedRole).queue();
            }
        }
        return punishment;
    }

    /**
     * Punish a member
     *
     * @param user   The user to punish
     * @param issuer The issuer of the punishment
     * @param guild  The guild the punishment is being issued in
     * @param type   The type of punishment
     * @param reason The reason for the punishment
     * @return The punishment issued
     */
    public Punishment punishMember(BatUser user, BatUser issuer, BatGuild guild, PunishmentType type, String reason) {
        return this.punishMember(user, issuer, guild, type, reason, -1);
    }

    /**
     * Temporarily punish a member
     *
     * @param user   The user to punish
     * @param issuer The issuer of the punishment
     * @param guild  The guild the punishment is being issued in
     * @param type   The type of punishment
     * @param reason The reason for the punishment
     * @param length The length of the punishment
     * @return The punishment issued
     */
    public Punishment temporaryPunish(BatUser user, BatUser issuer, BatGuild guild, PunishmentType type, String reason, long length) {
        return this.punishMember(user, issuer, guild, type, reason, length);
    }

    /**
     * Remove a punishment from a user
     *
     * @param user   The user to remove the punishment from
     * @param issuer The issuer of the punishment removal
     * @param guild  The guild the punishment is being removed from
     * @param type   The type of punishment
     * @param reason The reason for the punishment removal
     */
    public Punishment removePunishment(BatUser user, BatUser issuer, BatGuild guild, PunishmentType type, String reason) {
        Punishment punishment = this.getLatestActivePunishment(user, type);
        if (punishment == null && type != PunishmentType.BAN) {
            return null;
        }
        if (punishment == null) { // In case they are unbanning a user that wasn't banned by the bot
            punishment = new Punishment(type, reason, issuer.getId(), new Date(), null, false);
        }

        switch (type) {
            case BAN -> guild.getDiscordGuild().unban(user.getDiscordUser()).queue();
            case MUTE -> {
                Role mutedRole = this.findOrCreateMutedRole(guild);
                guild.getDiscordGuild().removeRoleFromMember(user.getDiscordUser(), mutedRole).queue();
            }
        }

        DescriptionBuilder descriptionBuilder = new DescriptionBuilder("%s Removed".formatted(EnumUtils.getEnumName(type)))
                .appendLine("User: %s".formatted(user.getDiscordUser().getAsMention()), true)
                .appendLine("Issuer: %s".formatted(issuer.getDiscordUser().getAsMention()), true)
                .appendLine("Reason: `%s`".formatted(reason), true);
        LogFeature.INSTANCE.sendLog(guild, LogType.PUNISHMENT_REMOVED_OR_EXPIRED, EmbedUtils.successEmbed()
                .setDescription(descriptionBuilder.build())
                .build());

        user.getDiscordUser().openPrivateChannel().queue(channel -> {
            DescriptionBuilder descriptionBuilder1 = new DescriptionBuilder(null);
            descriptionBuilder1.appendLine("🛡️ Your **%s** in **%s** has been removed".formatted(
                    EnumUtils.getEnumName(type),
                    guild.getDiscordGuild().getName()
            ), false);
            try {
                channel.sendMessage(descriptionBuilder1.build()).complete();
            } catch (Exception ignored) {
            } // Ignore since the user has DMs disabled
        });

        punishment.setExpired(true); // Expire the punishment
        return punishment;
    }

    /**
     * Gets the latest punishment for a user
     *
     * @param user The user to get the latest punishment for
     * @param type The type of punishment
     * @return The latest punishment
     */
    public Punishment getLatestActivePunishment(BatUser user, PunishmentType type) {
        List<Punishment> punishments = this.punishments.get(user.getId());
        if (punishments == null) {
            return null;
        }
        return punishments.stream()
                .filter(punishment -> punishment.getType() == type && !punishment.isExpired())
                .max(Comparator.comparing(Punishment::getIssuedDate))
                .orElse(null);
    }

    /**
     * Gets the punishments for a user
     *
     * @param user The user to get the punishments for
     * @return The punishments
     */
    public List<Punishment> getPunishments(BatUser user) {
        return this.punishments.getOrDefault(user.getId(), new ArrayList<>()).stream()
                .sorted(Comparator.comparing(Punishment::isExpired).reversed().thenComparing(Punishment::getIssuedDate).reversed())
                .toList();

    }

    /**
     * Gets the muted role for the Guild, if it
     * doesn't exist the bot will create one
     *
     * @param guild The guild to get the muted role for
     * @return The muted role
     */
    public Role findOrCreateMutedRole(BatGuild guild) {
        Guild discordGuild = guild.getDiscordGuild();
        Role mutedRole = discordGuild.getRolesByName("Muted", true).stream().findFirst().orElse(null);
        if (mutedRole == null) {
            return discordGuild.createRole()
                    .setName("Muted")
                    .complete();
        }
        return mutedRole;
    }

    /**
     * Sets the muted role permissions for a channel
     *
     * @param guild   the guild to get the muted role from
     * @param channel the channel to set the permissions for
     */
    public void upsetMutedRolePermissions(BatGuild guild, GuildChannel channel) {
        PunishmentProfile profile = guild.getPunishmentProfile();
        Role mutedRole = profile.findOrCreateMutedRole(guild);
        assert mutedRole != null;

        if (channel instanceof TextChannel textChannel) {
            textChannel.upsertPermissionOverride(mutedRole)
                    .deny(Permission.MESSAGE_SEND)
                    .queue();
            return;
        }
        if (channel instanceof VoiceChannel voiceChannel) {
            voiceChannel.upsertPermissionOverride(mutedRole)
                    .deny(Permission.VOICE_SPEAK)
                    .queue();
        }
    }

    /**
     * Ensures that the muted role permissions are
     * set up for all channels
     *
     * @param guild the guild to check
     */
    public void ensureMutedPermissionsAreSetup(BatGuild guild) {
        Role mutedRole = this.findOrCreateMutedRole(guild);
        for (GuildChannel channel : guild.getDiscordGuild().getChannels()) {
            for (PermissionOverride override : channel.getPermissionContainer().getRolePermissionOverrides()) {
                Role role = override.getRole();
                assert role != null;
                if (role.getId().equals(mutedRole.getId())) {
                    return;
                }
            }
            this.upsetMutedRolePermissions(guild, channel);
        }
    }

    /**
     * Inform the user that the punishment was applied
     *
     * @param event      The event to respond to
     * @param type       The type of punishment
     * @param targetUser The user that was punished
     * @param reason     The reason for the punishment
     * @param length     The length of the punishment
     */
    public void punishmentResponse(SlashCommandInteraction event, PunishmentType type, BatUser targetUser, String reason, boolean remove, long length) {
        DescriptionBuilder description = new DescriptionBuilder("Successfully %s%s %s".formatted(
                remove ? "Un-" : "",
                type.getPastTense(),
                targetUser.getDiscordUser().getAsMention()
        ));
        description.appendLine("Reason: `%s`".formatted(reason == null ? "No reason provided" : reason), true);
        if (length != -1) {
            description.appendLine("Length: %s".formatted(TimeUtils.format(length)), true);
        }
        event.replyEmbeds(EmbedUtils.successEmbed()
                .setDescription(description.build())
                .build()
        ).queue();
    }

    /**
     * Check if a user can be punished
     *
     * @param guild      The guild to check
     * @param issuer     The issuer of the punishment
     * @param targetUser The user to punish
     * @return If the user can be punished
     */
    public boolean canPunish(BatGuild guild, BatUser issuer, BatUser targetUser) {
        Member owner = guild.getDiscordGuild().getOwner();
        assert owner != null;
        if (targetUser == null) {
            return false;
        }

        // Check if the target user is the owner
        if (owner.getId().equals(targetUser.getId())) {
            return false;
        }

        // Check if the target user is the bot
        if (targetUser.getId().equals(guild.getDiscordGuild().getSelfMember().getId())) {
            return false;
        }

        // Check if the target user is the same as the issuer
        if (targetUser.getId().equals(issuer.getId())) {
            return false;
        }

        // Check if the target user has a higher role than the bot
        Member member = guild.getDiscordGuild().getMemberById(targetUser.getId());
        assert member != null;
        if (RoleUtils.hasHigherRole(member, guild.getDiscordGuild().getSelfMember())) {
            return false;
        }

        return true;
    }

    @Override
    public void load(Document document, Gson gson) {
        Document punishmentsDocument = (Document) document.getOrDefault("punishments", new Document());
        for (String key : punishmentsDocument.keySet()) {
            List<Document> punishmentList = punishmentsDocument.getList(key, Document.class, new ArrayList<>());
            List<Punishment> punishments = new ArrayList<>();
            for (Document punishmentDocument : punishmentList) {
                punishments.add(new Punishment(
                        PunishmentType.valueOf(punishmentDocument.getString("type")),
                        punishmentDocument.getString("reason"),
                        punishmentDocument.getString("issuer"),
                        punishmentDocument.getDate("issuedDate"),
                        punishmentDocument.getDate("endDate"),
                        punishmentDocument.getBoolean("expired", false)
                ));
            }
            this.punishments.put(key, punishments);
        }
    }

    @Override
    public Document serialize(Gson gson) {
        Document document = new Document();
        Document punishmentsDocument = new Document();
        for (String key : this.punishments.keySet()) {
            List<Document> punishmentList = new ArrayList<>();
            for (Punishment punishment : this.punishments.get(key)) {
                Document punishmentDocument = new Document();
                punishmentDocument.append("type", punishment.getType().name());
                punishmentDocument.append("reason", punishment.getReason());
                punishmentDocument.append("issuer", punishment.getIssuer());
                punishmentDocument.append("issuedDate", punishment.getIssuedDate());
                punishmentDocument.append("endDate", punishment.getEndDate());
                punishmentDocument.append("expired", punishment.isExpired());
                punishmentList.add(punishmentDocument);
            }
            punishmentsDocument.append(key, punishmentList);
        }
        document.append("punishments", punishmentsDocument);
        return document;
    }

    @Override
    public void reset() {
        this.punishments.clear();
    }
}
