package cc.fascinated.bat.premium;

import cc.fascinated.bat.common.Serializable;
import com.google.gson.Gson;
import lombok.Getter;
import lombok.NoArgsConstructor;
import lombok.Setter;
import org.bson.Document;

import java.util.Calendar;
import java.util.Date;

/**
 * @author Fascinated (fascinated7)
 */
@Getter
@Setter
@NoArgsConstructor
public class PremiumProfile extends Serializable {
    /**
     * The time the premium was activated
     */
    private Date activatedAt;

    /**
     * The time the premium expires
     */
    private Date expiresAt;

    /**
     * The type of premium
     */
    private Type type;

    /**
     * Checks if the guild has premium
     *
     * @return whether the guild has premium
     */
    public boolean hasPremium() {
        return this.type == Type.INFINITE || (this.expiresAt != null && this.expiresAt.after(new Date()));
    }

    /**
     * Adds a month to the premium time
     */
    public void addTime(int months) {
        if (this.type == null) { // If the type is null, set it to monthly
            this.type = Type.MONTHLY;
        }
        if (this.expiresAt == null) {
            this.expiresAt = new Date();
        }
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(new Date());
        calendar.add(Calendar.MONTH, months);
        this.expiresAt = calendar.getTime();
        this.type = Type.MONTHLY;
    }

    /**
     * Adds a month to the premium time
     */
    public void addTime() {
        addTime(1);
    }

    /**
     * Adds infinite time to the premium
     */
    public void addInfiniteTime() {
        this.type = Type.INFINITE;
        this.expiresAt = null;
        this.activatedAt = new Date();
    }

    /**
     * Removes the premium from the guild
     */
    public void removePremium() {
        this.activatedAt = null;
        this.expiresAt = null;
        this.type = null;
    }

    /**
     * Checks if the premium is infinite
     *
     * @return whether the premium is infinite
     */
    public boolean isInfinite() {
        return this.type == Type.INFINITE;
    }

    /**
     * Checks if the premium has expired
     *
     * @return whether the premium has expired
     */
    public boolean hasExpired() {
        return this.expiresAt != null && this.expiresAt.before(new Date());
    }

    /**
     * The premium type for the guild
     */
    public enum Type {
        INFINITE,
        MONTHLY
    }

    @Override
    public void load(Document document, Gson gson) {
        this.activatedAt = (Date) document.getOrDefault("activatedAt", new Date());
        this.expiresAt = (Date) document.getOrDefault("expiresAt", null);
        this.type = document.getString("type") != null ? Type.valueOf(document.getString("type")) : null;
    }

    @Override
    public Document serialize(Gson gson) {
        Document document = new Document();
        document.put("activatedAt", this.activatedAt);
        document.put("expiresAt", this.expiresAt);
        document.put("type", this.type != null ? this.type.name() : null);
        return document;
    }

    @Override
    public void reset() {
        this.activatedAt = null;
        this.expiresAt = null;
        this.type = null;
    }
}
